#! /bin/csh -f
#
#	Experimental OASIS script
#
#
# set this to wherever your awk program is
alias nawk /usr/bin/nawk
nawk 'BEGIN{print}' >& /dev/null
if($status) alias nawk awk


set script  = ./scripts/mlphare.com
set mtzfile = ./mtz/all.mtz
set F1      = ""		# these can be assigned on the command line
set SIGF1   = ""		# but are normally just read in from the mtz
set F2      = ""
set SIGF2   = ""
set PHI     = ""

# these might be important
set Ee      = Se
set sites   = ""		# use the default
set fpp     = ""
set lambda  = "0.9794"		# only needed if fpp is unknown
set outfile = ./oasised.mtz

set tempfile = oasis_temp

if($#argv == 0) goto Help
goto Setup
# scan command line for:
# mlphare script (and read in atom coordinates)
# new element name
Help:
cat << EOF

usage: $0 mlphare.com FP DANO sad.mtz $Ee 6.32e

where:
mlphare.com	- is an mlphare script, containing the refined sites
sad.mtz		- is an MTZ containing FP and DANO for your SAD data
FP		- the full protein F data set
DANO		- the anomalous difference data set
$Ee		- is the anomalous scatterer
6.32e		- is the expected f" value (in electron equivalents)

defaults:
FP      will default to the first F found in sad.mtz
DANO    will default to the first D found in sad.mtz
f"      will default to the highest anomalous occupancy in mlphare.com
sad.mtz will default to the mtz file used in mlphare.com

note: all mlphare.com really need be is a list of:
ATOM ANO x y z 0 occ
where x,y,z are fractional coordinates, and occ is a "relative" occupancy
EOF

exit 2
ReturnFromSetup:

if(! -e "$mtzfile") then
    echo "ERROR: $mtzfile does not exist! "
    goto Help
endif

#####################################################################################
#####################################################################################
#####################################################################################
cat << EOF >! ${tempfile}oasis.in
CEL $CELL
HCO $EE $cell_sites
ANO $EE $fpp
LCE 7
FIT
OAS
$usePHI
LABIN  F1=$F1 SIGF1=$SIGF1 F2=$F2 SIGF2=$SIGF2 $TPHI
LABOUT F1=$F1 SIGF1=$SIGF1 PHI=PHIOAS W=FOMOAS
EOF
# append the site list
cat ${tempfile}sites >> ${tempfile}oasis.in
echo "END"           >> ${tempfile}oasis.in

# display this, so user can actually see it! 
cat << EOF
Running "oasis" with $sites $Ee sites from $script on
$F1 and $F2 from $mtzfile
Assuming f" of $Ee is $fpp at the wavelength used for $F1
EOF
if("$PHI" != "") echo "$PHI will be used for comparison only"
echo "-------------------------------------------------------"
echo "OASIS input:"
cat ${tempfile}oasis.in
echo "-------------------------------------------------------"

# now, actually run oasis:
cat ${tempfile}oasis.in |\
oasis hklin $mtzfile hklout $outfile 
if($status) then
    echo "Woops! See above for what went wrong."
    exit 9
endif

echo "$outfile is ready. "

# clean up
rm -f ${tempfile}sites >& /dev/null
rm -f ${tempfile}oasis.in >& /dev/null

exit


Setup:
#####################################################################################
#####################################################################################
#####################################################################################
set TPHI   = ""
set usePHI = ""
set EE

# scan command line for args
foreach arg ( $* )
    # warn about probable mispellings
    if("$arg" =~ *.mtz) then
	if(! -e "$arg") then
	    echo "WARNING: $arg does not exist! "
	    continue
	endif
	set mtzfile = "$arg"
	continue
    endif
    
    if("$arg" =~ *phare*) then
	if(! -e "$arg") then
	    echo "WARNING: $arg does not exist! "
	    continue
	endif
	grep "ATOM" "$arg" >& /dev/null
	if($status) then
	    echo "WARNING: no atoms in $arg"
	    continue
	endif 
	set script = "$arg"
	continue
    endif
    
    if(("$arg" =~ [A-Z][a-y])||("$arg" =~ [HBCNOFPSKVYIWU])) then
	# might as well consider this an element
	set temp = "$arg"
	if($?CLIBD) then
	    # check the CCP4 atom database
	    set temp = `nawk -v arg=$arg 'NF==1 && toupper($1)==toupper(arg){print; exit}' $CLIBD/atomsf.lib`
	endif
	if("$temp" != "") then
	    set Ee = "$arg"
	    continue
	endif
    endif
end

grep "ATOM" $script >& /dev/null
if($status) then
    echo "ERROR! no atoms in $script"
    goto Help
endif

if(! -e "$mtzfile") then
    # see if we can retrieve the script's mtz
    set mtzfiles = `nawk 'BEGIN{RS=" "} {gsub("[\042\047]"," ",$0); print}' $script | nawk '/\.mtz$/{print}'`
    
    foreach mtz ( $mtzfile $mtzfiles )
	echo "head" | mtzdump hklin $mtz >& /dev/null
	if(! $status) then
	    echo "reading $mtz"
	    set mtzfile = $mtz
	    break
	endif
    end
endif

if(! -e "$mtzfile") then
    echo "ERROR: $mtzfile does not exist! "
    goto Help
endif

# read important constants from the mtz header
echo "go" | mtzdump HKLIN $mtzfile >! ${tempfile}mtzdump
set CELL  = `nawk '/Cell Dimensions/{getline;getline;print}' ${tempfile}mtzdump`
set SG    = `nawk '/Space group/{print $5}' ${tempfile}mtzdump`
set SGnum = ` nawk '/Space group/{print $NF+0}' ${tempfile}mtzdump `
set SG = ` nawk -F "[\047]" '/Space group/{print $2}' ${tempfile}mtzdump `
set SG = ` nawk -v num=$SGnum '$1==num && NF>5{print $4}' ${CLIBD}/symop.lib `
set hiRES = `nawk '/Resolution Range/{getline;getline;print $6}' ${tempfile}mtzdump`

if($#CELL != 6) then
    echo "ERROR: unable to read $mtzfile"
    goto Help
endif

# get data column label names from the mtz file
nawk 'NF>3' ${tempfile}mtzdump |\
nawk '$(NF-1)=="F"{print "F", $NF}\
      $(NF-1)=="D"{print "D", $NF}\
      $(NF-1)=="P"{print "P", $NF}\
      $(NF-1)=="Q"{print "S", $NF}' |\
nawk '/^F/{++n} {printf "%s", $1; \
       if($1=="S") printf "%s", last;\
printf " %d %s\n",n, $2; last=$1}' |\
nawk '$1=="F"{F[$2]=$NF} $1=="SF"{SF[$2]=$NF} \
      $1=="D"{D[$2]=$NF} $1=="SD"{SD[$2]=$NF} \
      $1=="P"{P=P $NF} \
      END{for(i in F){\
	print i, F[i], SF[i], D[i],SD[i], P, " ";}}' |\
sort -n >! ${tempfile}datasets


# one last pass through command line
# allow user overrides of all internal variables
set i = 0
echo -n "" >! ${tempfile}userlabels
while( $i < $#argv )
    @ i = ( $i + 1 )
    @ nexti = ( $i + 1 )
    @ lasti = ( $i - 1 )
    if($nexti > $#argv) set nexti = $#argv
    if($lasti < 1) set lasti = 1
    set arg = "$argv[$i]"
    
    # see if a dataset label was given
    grep " $arg " ${tempfile}datasets >& /dev/null
    if(! $status) then
	if($?NO) then
	    # user doesn't want this label
	    # filter it out of the input files
	    egrep -v " $arg " ${tempfile}datasets >! ${tempfile}
	    mv ${tempfile} ${tempfile}datasets
	    unset NO
	else
	    # must want only this label?
	    cat ${tempfile}mtzdump |\
	    nawk -v label=$arg 'NF>2 && $NF==label{print $NF}' |\
	    cat >> ${tempfile}userlabels
	endif
	continue
    endif
    
    # only look at non-file words now
    if(! -e "$arg") then
	if("$arg" =~ [0-9]*) then
	    # we have a number
	    if(("$arg" =~ *A)||("$argv[$nexti]" == "A")) then
		# user-preferred resolution limits
		set temp = `echo "$arg" | nawk 'BEGIN{FS="-"} $1+0 > 0.1{print $1+0} $2+0 > 0.1{print $2+0}'`
		if($#temp != 1) then
		    set temp = `echo $temp | nawk '$1>$2{print $1, $2} $2>$1{print $2, $1}'`
		    if($#temp == 2) then
			set loRES = "$temp[1]"
			set hiRES = "$temp[2]"
		    endif
		else
		    # this isn't used, but...
		    if("$temp" != "") set hiRES = "$temp"
		endif
		unset NO
		continue
	    endif
	    
	    # maybe setting f" value?
	    if(("$arg" =~ *e)||("$argv[$nexti]" == "e")) then
		set temp = `echo "$arg" | nawk '$1+0>0.1 && $1+0<100{print $1+0}'`
		if($#temp == 1) then
		    set fpp = "$temp"
		    unset NO
		    continue
		endif
	    endif
	endif
	
	# allow "NO" logic to carry through
	unset NO
	if(("$arg" == "no")||("$arg" == "not")) set NO
	if(("$arg" == "don't")||("$arg" == "ignore")) set NO
	if("$arg" == "except") set NO
    endif
end

rm -f ${tempfile}mtzdump >& /dev/null

#######################################################
# see if user specified particular labels
set temp = `cat ${tempfile}userlabels | wc -l`
if($temp != 0) then
    # turn the "user" labels into real label files
    cat ${tempfile}userlabels ${tempfile}datasets |\
    nawk 'NF==1{++n; label[n]=$NF}\
          NF>1 {for(i in label) for(j=2;j<=NF;++j){\
		if(label[i]==$j) print i, $2,$3,$4,$5;break}}' |\
    sort -n >! ${tempfile}
    mv ${tempfile} ${tempfile}datasets >& /dev/null
endif
if(($temp == 0)&&("$F1" != "")) then
    # apply the labels from the top of this script?
    echo "$F1 $F2 $SIGF1 $SIGF2 $PHI" | nawk '{for(i=1;i<=NF;++i) print $i}' >! ${tempfile}userlabels
    cat ${tempfile}userlabels ${tempfile}datasets |\
    nawk 'NF==1{++n; label[n]=$NF}\
          NF>1 {for(i in label) for(j=2;j<=NF;++j){\
		if(label[i]==$j) print i, $2,$3,$4,$5;break}}' |\
    sort -n >! ${tempfile}
    set temp = `cat ${tempfile} | wc -l`
    if($temp != 0) mv ${tempfile} ${tempfile}datasets >& /dev/null    
endif
rm -f ${tempfile}userlabels >& /dev/null

###############################################################
# assign actual "LABIN" cards for oasis for each dataset
if("$F1" == "")    set F1 = `head -1 ${tempfile}datasets | nawk '{print $2}'`
if("$F2" == "")    set F2 = `head -1 ${tempfile}datasets | nawk '{print $4}'`
if("$SIGF1" == "") set SIGF1 = `head -1 ${tempfile}datasets | nawk '{print $3}'`
if("$SIGF2" == "") set SIGF2 = `head -1 ${tempfile}datasets | nawk '{print $5}'`
grep " $PHI " ${tempfile}datasets >& /dev/null
if($status) set PHI = ""
if("$PHI" == "") set PHI = `head -1 ${tempfile}datasets | nawk '{print $6}'`
if("$PHI" != "") then
    set TPHI = "TPHI=$PHI"
    set usePHI = "PHI"
endif

rm -f ${tempfile}datasets >& /dev/null

# oasis needs this to be uppercase
set EE = `echo "$Ee" | nawk '{print toupper($1)}'`

###############################################################
# convert MLphare sites to OASIS sites
set maxocc = `nawk '$1 ~ /^ATOM/{print $7}' $script | sort -n | tail -1`
cat $script |\
nawk -v Ee=$EE -v norm=$maxocc 'BEGIN{pos = "POS"}\
 $1 ~ /^ATOM/{++i; printf "%-4s %2s %7s %7s %7s %3d %7.5f\n", pos, toupper(Ee), $3, $4, $5, i, $7/norm; pos=""}' |\
cat >! ${tempfile}sites
set sites = `cat ${tempfile}sites | wc -l`

# need number in cell
set ASU_per_CELL = `nawk -v SG=$SG '$4==SG{print $3}' $CLIBD/symop.lib | head -1`
set cell_sites = `echo "$sites $ASU_per_CELL" | nawk '{print $1 * $2}'`

# this is probably the best indicator of f" anyway
if("$fpp" == "") set fpp = "$maxocc"

if("$fpp" == "") then
    # use crossec to calculate this from the wavelength?
    set fpp = 1
endif


goto ReturnFromSetup


exit
#############################################
# the future?

read solve.status (where do we get $Ee?)
