#! /bin/csh -f
#
#
#	revise.com 
#
#	calculate an estimate of FM for the heavy atoms alone
#	using CCP4's "revise" program
#
#	FM should give cleaner Pattersons, difference Fouriers,
#	and direct methods.
#
#	Do NOT combine differences from data sets expected to have
#	different heavy atom locations.  That would be silly.
#
#
# set this to wherever your awk program is
alias nawk /usr/bin/nawk
nawk 'BEGIN{print}' >& /dev/null
if($status) alias nawk awk

# defaults (can be overriden by command-line input)
set mtzfile     = "./mtz/all.mtz"	    # MAD data set
set fp          = ""			# set these if you know what they are
set fpp         = ""			# set these if you know what they are
set Fs		= "Frem1 Fpeak"	# overriden by contents of mtz
set wavelengths = "1.12713 0.97959"	# only important if fp/fpp are unassiged
set Ee		= "Se"			    # only important if fp/fpp are unassiged

set outfile     = "./FM.mtz"		# contains FH, SIGFH
set shelxfile   = "./fm"		# same thing, shelx format
set pattfile    = "./FM_Patt.map"	# Combined Patterson map

set tempfile = ./revise_temp$$


# initialize internal variables
set loRES = 1000
set hiRES = ""

set DATA_cutoff = 0	# sigma cutoff      
set MAX_dano 	= 0.5	# upper Dano cutoff 
set MAX_diso	= 0.1	# upper Diso cutoff 

if($#argv == 0) goto Help

# this procedure (re)sets most of the above variables
# from either the provided files, or the command line
goto Setup

Help:
cat << EOF

usage: $0 $mtzfile $Fs

where:
$mtzfile	- contains at least two sets of F and DANO 
		  you want to combine


EOF

exit 2
#
#   This procedure (at the bottom of the script) does the following
#   1) scan the command line for the mtz file
#   2) set the CELL, SG, and other variables
#   3) generate dataset name list:
#      ${tempfile}LABIN and ${tempfile}waves
Return_from_Setup:


################################################################
#   initial report on intended program flow
################################################################
echo "running revise on $mtzfile"
echo "with:"

Crossec:
################################################################
if("$fp" != "") goto Revise

# make a lookup table for recognizing sorted wavelengths
echo "$wavelengths" | nawk '{for(i=1;i<=NF;++i) print i, $i, "order"}' |\
sort -n +1 |\
cat >! ${tempfile}waveorder

# calculate f' and f" of $Ee at the provided wavelengths
crossec << EOF >! ${tempfile}crossec.log
ATOM $Ee
NWAVE $#wavelengths $wavelengths
END
EOF
# log will be printed out and deleted later

# condense crossec output to vital info only
cat ${tempfile}crossec.log |\
nawk 'NF==4 && $NF !~ /[^0-9\.]/' |\
sort -n +1 |\
cat >! ${tempfile}sortedwaves

# now put the wavelengths back in the order we originally had them
cat ${tempfile}waveorder ${tempfile}sortedwaves |\
nawk '$NF=="order"{++i; order[i]=$1} \
      $NF!="order"{++n; print order[n], $2, $3, $4}' |\
sort -n >! ${tempfile}fp_fdp

# load f' and f" into variables (in dataset order)
set fp  = `nawk '{print $3}' ${tempfile}fp_fdp`
set fpp = `nawk '{print $4}' ${tempfile}fp_fdp`

# clean up
rm -f ${tempfile}fp_fdp >& /dev/null
rm -f ${tempfile}waveorder >& /dev/null
rm -f ${tempfile}sortedwaves >& /dev/null




Revise:
################################################################
# make a table of f' f" values (vs dataset number)
echo "$fp" | nawk '{for(i=1;i<=NF;++i) print i, $i, "fp"}' |\
cat >! ${tempfile}fp
echo "$fpp" | nawk '{for(i=1;i<=NF;++i) print i, $i, "fpp"}' |\
cat >! ${tempfile}fpp

# same thing for wavelengths (vs dataset number)
echo "$wavelengths" | nawk '{for(i=1;i<=NF;++i) printf "%d %.4f wave\n", i, $i}' |\
cat >! ${tempfile}waves

# make a table of datasets (vs dataset number)
cat ${tempfile}LABIN |\
nawk '/^FPH/{split($1,w,"="); i=substr($1,4)+0; print i, w[2], "set"}' |\
cat >! ${tempfile}datasets

# combine all this information into revise "WAVE" keywords
cat ${tempfile}datasets ${tempfile}waves ${tempfile}fp ${tempfile}fpp |\
nawk '$NF=="fp"{fp[$1]=$2} $NF=="fpp"{fpp[$1]=$2} \
      $NF=="set"{set[$1]=$2} $NF=="wave"{wave[$1]=$2} \
      END{for(i in set) \
      print "WAVE", i, "LAM", wave[i], "FPR", fp[i], "FDP", fpp[i]}' |\
sort -n +1 >! ${tempfile}WAVE

# clean up 
rm -f ${tempfile}datasets >& /dev/null
rm -f ${tempfile}waves    >& /dev/null
rm -f ${tempfile}fp       >& /dev/null
rm -f ${tempfile}fpp      >& /dev/null

# display f' f" stuff for user:
echo 'dataset wavelength  f'"'"'     f"   '
cat ${tempfile}LABIN ${tempfile}WAVE |\
nawk '/^FPH/{split($1,w,"="); i=substr($1,4)+0; F[i]=w[2]} \
/^WAVE/{printf "%7s  %7.5fA  %s %s\n",F[$2],$4,$6,$8}'
echo ""
echo ""
test -t 1
if(! $status) then
    echo "is this okay? [Yes]"
    echo -n "-> "
    set temp = "$<"
    if("$temp" =~ [Nn]*) then
	echo "please either change the order of your data sets (on the command line)"
	echo "or edit $0 to enter in the proper f' and f"\"" values."
	rm -f ${tempfile}LABIN ${tempfile}WAVE >& /dev/null
	exit 3
    endif
endif

# print this out for user's benifit
if(-e ${tempfile}crossec.log) then
    cat ${tempfile}crossec.log
    rm -f ${tempfile}crossec.log >& /dev/null
endif

################################################################
# now actually run revise! 
################################################################
revise hklin $mtzfile hklout ${tempfile}revised.mtz << eof
TITLE  revised $mtzfile

# read in Fs:
@${tempfile}LABIN

LABO FM=FM SIGFM=SIGFM


# read in WAVE cards
@${tempfile}WAVE

EXCL RISO $MAX_diso RANO $MAX_dano SIGM $DATA_cutoff

END
eof

# strip off everything but FM
rm -f $outfile >& /dev/null
cad hklin1 ${tempfile}revised.mtz hklout $outfile << EOF
LABIN FILE 1 E1=FM E2=SIGFM
#LABOUT ALLIN
EOF
if((! $status)&&(-e $outfile)) then
    echo "$outfile is ready."
    echo "calculated from $mtzfile using:"
    echo 'dataset wavelength  f'"'"'     f"   '
    cat ${tempfile}LABIN ${tempfile}WAVE |\
    nawk '/^FPH/{split($1,w,"="); i=substr($1,4)+0; F[i]=w[2]} \
    /^WAVE/{printf "%7s  %7.4fA  %s %s\n",F[$2],$4,$6,$8}'
endif

# clean up a bit
rm -f ${tempfile}WAVE >& /dev/null
rm -f ${tempfile}LABIN >& /dev/null
rm -f ${tempfile}revised.mtz >& /dev/null

Shelx_format:
################################################################
if("$shelxfile" == "") goto Patterson

# dump FM out in shelx format too.
mtz2various HKLIN $outfile HKLOUT ${tempfile}.hkl << EOF-shelx
OUTPUT SHELX
LABIN FP=FM SIGFP=SIGFM
END
EOF-shelx
nawk '! /^[A-Z]/ && $5+0!=0' ${tempfile}.hkl >! ${shelxfile}.hkl
nawk '/^[A-Z]/' ${tempfile}.hkl >! ${shelxfile}.ins 
cat << EOF >> ${shelxfile}.ins 
SFAC Se
UNIT 5
TREF
HKLF 3
EOF
rm -f ${tempfile}.hkl ${shelxfile}.ins

if((! $status)&&(-e "${shelxfile}.hkl")) echo "${shelxfile}.hkl is the SHELX version of $outfile."


Patterson:
################################################################
if("$pattfile" == "") goto Clean_up

# use scaleit to get the recommended DIFF
scaleit hklin $outfile << EOF >! ${tempfile}scaleit
analyze
labin FP=FM SIGFP=SIGFM FPH1=FM SIGFPH1=SIGFM
SCALE FPH1 0.000001
END
EOF
if(! $?DIFF) set DIFF
set DIFF = `nawk '/acceptable differences/{print $NF}' ${tempfile}scaleit`
rm -f ${tempfile}scaleit >& /dev/null
if("$DIFF" == "") set DIFF = 1000

# calculate the Patterson map (if desired)
fft HKLIN $outfile MAPOUT ${tempfile}FM.map << EOF-fft
PATTERSON
RESO $loRES $hiRES
LABIN F1=FM SIG1=SIGFM F2=FM SIG2=SIGFM
SCALE F2 0.000001 0
EXCLUDE DIFF $DIFF
EXCLUDE SIG1 0
EOF-fft
peakmax MAPIN ${tempfile}FM.map XYZOUT ${tempfile}.pdb << EOF-pick
THRESHOLD RMS 3
NUMPEAKS 50
EOF-pick
# extend it to whole unit cell?
mapmask mapin ${tempfile}FM.map mapout "$pattfile" << EOF
scale sigma
#xyzlim 0 1 0 1 0 1
EOF
if((! $status)&&(-e "$pattfile")) echo "$pattfile is ready."
rm -f ${tempfile}FM.map >& /dev/null


Clean_up:
################################################################
# clean up


rm -f ${tempfile}.pdb   >& /dev/null


exit






Setup:
#################################################

  ####   ######   #####  #    #  #####
 #       #          #    #    #  #    #
  ####   #####      #    #    #  #    #
      #  #          #    #    #  #####
 #    #  #          #    #    #  #
  ####   ######     #     ####   #

#################################################
#
#   gather information on:
#    mtz file
#    data sets
#    resolution limits
#    sigma cuttoff (for map generation)
#    difference cutoff
#
##################################################
# make multiword variables out of these
set Fs		= ( $Fs )
set wavelengths = ( $wavelengths )
set fp          = ( $fp )
set fpp         = ( $fpp )

# scan the command line for files
foreach arg ( $* )
    # warn about probable mispellings
    if("$arg" =~ *.mtz) then
	if(! -e "$arg") then
	    echo "WARNING: $arg does not exist! "
	    continue
	endif
	set mtzfile = "$arg"
    endif
    
    if("$arg" =~ [A-Z][a-y]) then
	# might as well consider this an element
	set temp = "$arg"
	if($?CLIBD) then
	    # check the CCP4 atom database
	    set temp = `nawk -v arg=$arg 'NF==1 && toupper($1)==toupper(arg){print; exit}' $CLIBD/atomsf.lib`
	endif
	if("$temp" != "") set Ee = "$arg"
	continue
    endif
end


# now all filenames have been initialized

if(! -e "$mtzfile") goto Help

##################################################
# get crystal and dataset information from the mtz file
echo "go" | mtzdump HKLIN $mtzfile >! ${tempfile}mtzdump
set CELL  = `nawk '/Cell Dimensions/{getline;getline;print}' ${tempfile}mtzdump`
set SG    = `nawk '/Space group/{print $5}' ${tempfile}mtzdump`
set SGnum = ` nawk '/Space group/{print $NF+0}' ${tempfile}mtzdump `
set SG = ` nawk -F "[\047]" '/Space group/{print $2}' ${tempfile}mtzdump `
set SG = ` nawk -v num=$SGnum '$1==num && NF>5{print $4}' ${CLIBD}/symop.lib `
set hiRES = `nawk '/Resolution Range/{getline;getline;print $6}' ${tempfile}mtzdump`

# get column label names from the mtz file
nawk 'NF>3' ${tempfile}mtzdump |\
nawk '$(NF-1)=="F"{print "F", $NF}\
      $(NF-1)=="D"{print "D", $NF}\
      $(NF-1)=="Q"{print "S", $NF}' |\
nawk '/^F/{++n} {printf "%s", $1; \
       if($1=="S") printf "%s", last;\
printf " %d %s\n",n, $2; last=$1}' |\
nawk '$1=="F"{F[$2]=$NF} $1=="SF"{SF[$2]=$NF} \
      $1=="D"{D[$2]=$NF} $1=="SD"{SD[$2]=$NF} \
      END{for(i in F){\
	print i, F[i], SF[i], D[i],SD[i];}}' |\
sort -n >! ${tempfile}datasets


# check extent of available data
set temp = `cat ${tempfile}datasets |& wc -l`
if($temp < 2) then
    # this is useless, revise won't run. bail out now
    rm -f ${tempfile}datasets >& /dev/null
    echo "ERROR: no usable data in ${mtzfile}! "
    set mtzfile = ""
    goto Help
endif


# one last pass through command line
# allow user overrides of all internal variables
set i = 0
echo -n "" >! ${tempfile}userlabels
while( $i < $#argv )
    @ i = ( $i + 1 )
    @ nexti = ( $i + 1 )
    @ lasti = ( $i - 1 )
    if($nexti > $#argv) set nexti = $#argv
    if($lasti < 1) set lasti = 1
    set arg = "$argv[$i]"
    
    # see if a dataset label was given
    grep " $arg " ${tempfile}datasets >& /dev/null
    if(! $status) then
	if($?NO) then
	    # user doesn't want this label
	    # filter it out of the input files
	    egrep -v " $arg " ${tempfile}datasets >! ${tempfile}
	    mv ${tempfile} ${tempfile}datasets
	    continue
	else
	    # must want only this label?
	    cat ${tempfile}mtzdump |\
	    nawk -v label=$arg 'NF>2 && $NF==label{print $NF}' |\
	    cat >> ${tempfile}userlabels
	endif
    endif
    
    # only look at non-file words
    if(! -e "$arg") then
	if("$arg" =~ [0-9]*) then
	    # we have a number
	    if(("$arg" =~ *A)||("$argv[$nexti]" == "A")) then
		# user-preferred resolution limits
		set temp = `echo "$arg" | nawk 'BEGIN{FS="-"} $1+0 > 0.1{print $1+0} $2+0 > 0.1{print $2+0}'`
		if($#temp != 1) then
		    set temp = `echo $temp | nawk '$1>$2{print $1, $2} $2>$1{print $2, $1}'`
		    if($#temp == 2) then
			set loRES = "$temp[1]"
			set hiRES = "$temp[2]"
		    endif
		else
		    if("$temp" != "") set hiRES = "$temp"
		endif
	    endif
	    
	    if(("$arg" =~ *[Ss]igma)||("$argv[$nexti]" =~ [Ss]igma)) then
		set DATA_cutoff = `echo "$arg" | nawk '$1+0 > 0{print $1+0}'`
	    endif
	    
	    if(("$arg" =~ *[Dd]iff)||("$argv[$lasti]" =~ [Dd]iff)) then
		set MAX_diso = `echo "$arg" | nawk '$1+0 > 0{print $1+0}'`
		set MAX_dano = `echo "$arg" | nawk '$1+0 > 0{print $1+0}'`
	    endif
	    if(("$arg" =~ *[Dd]iso)||("$argv[$lasti]" =~ [Dd]iso)) then
		set MAX_diso = `echo "$arg" | nawk '$1+0 > 0{print $1+0}'`
	    endif
	    if(("$arg" =~ *[Dd]ano)||("$argv[$lasti]" =~ [Dd]ano)) then
		set MAX_dano = `echo "$arg" | nawk '$1+0 > 0{print $1+0}'`
	    endif
	endif
	
	# allow "NO" logic to carry through
	unset NO
	if(("$arg" == "no")||("$arg" == "not")) set NO
	if(("$arg" == "don't")||("$arg" == "ignore")) set NO
	if("$arg" == "except") set NO
    endif
end

rm -f ${tempfile}mtzdump >& /dev/null

#######################################################
# see if user specified particular labels
set temp = `cat ${tempfile}userlabels | wc -l`
if($temp != 0) then
    # turn the "user" labels into real label files
    cat ${tempfile}userlabels ${tempfile}datasets |\
    nawk 'NF==1{++n; label[n]=$NF}\
          NF>1 {for(i in label) for(j=2;j<=NF;++j){\
		if(label[i]==$j) print i, $2,$3,$4,$5;break}}' |\
    sort -n >! ${tempfile}
    mv ${tempfile} ${tempfile}datasets >& /dev/null
endif
if(($temp == 0)&&($#Fs != 0)) then
    # apply the labels from the top of the script?
    echo "$Fs" | nawk '{for(i=1;i<=NF;++i) print $i}' >! ${tempfile}userlabels
    cat ${tempfile}userlabels ${tempfile}datasets |\
    nawk 'NF==1{++n; label[n]=$NF}\
          NF>1 {for(i in label) for(j=2;j<=NF;++j){\
		if(label[i]==$j) print i, $2,$3,$4,$5;break}}' |\
    sort -n >! ${tempfile}
    set temp = `cat ${tempfile} | wc -l`
    if($temp != 0) mv ${tempfile} ${tempfile}datasets >& /dev/null    
endif


###############################################################
# create "LABIN" cards for revise for each dataset
cat ${tempfile}datasets |\
nawk '{print "FPH" $1 "=" $2, "SIGFPH" $1 "=" $3,\
             "DPH" $1 "=" $4, "SIGDPH" $1 "=" $5;}' |\
sort -n |\
nawk 'BEGIN{printf "%s", "LABIN"} {printf " -\n%s", $0} END{print ""}' |\
cat >! ${tempfile}LABIN


# convert the labin cards into an "Fs" variable
cat ${tempfile}LABIN |\
nawk '/^FPH/{i=substr($1,4)+0; split($1,w,"="); print i, w[2]}' |\
sort -n | nawk '{print $NF}' |\
cat >! ${tempfile}Fs
set Fs = `cat ${tempfile}Fs`
if($#Fs < 2) then
    echo "Sorry, revise needs at least two wavelengths."
    goto Help
endif

# make a table connecting Fs to wavelengths
echo "$wavelengths" |\
 nawk '{for(i=1;i<=NF;++i) print i,$i,"lambda"}' |\
cat >! ${tempfile}waves
exit

# clean up
rm -f ${tempfile}Fs >& /dev/null
rm -f ${tempfile}datasets >& /dev/null
rm -f ${tempfile}userlabels >& /dev/null

goto Return_from_Setup


exit
####################################################

The Future:
make the interface a little slicker, less wordy?
fix problems with single-difference data
use correlation instead of scale as a weight?
implement use of max_DANO, DATA_cutoff, etc. ?


fft HKLIN FM.mtz MAPOUT patt.map << EOF-fft
PATTERSON
RESO $loRES $hiRES
LABIN F1=FM SIG1=SIGFM
EOF-fft
peakmax MAPIN patt.map XYZOUT ${tempfile}.pdb << EOF-pick
THRESHOLD RMS 3
NUMPEAKS 50
EOF-pick
/max/jamesh/Develop/vmap patt.map &

set hiRES = 2
set loRES = 1000
set DIFF = 100
set SIG  = 0.0

foreach hiRES ( `nawk 'BEGIN{for(i=2;i<=6;i+=0.1) printf "%.1f ", i}'` )
echo -n "$hiRES "

foreach loRES ( `nawk 'BEGIN{for(i=3;i>=1.6;i-=0.05) printf "%.1f ", i^3}'` )
echo -n "$loRES "

foreach DIFF ( `nawk 'BEGIN{for(i=30;i>=2;i-=1) printf "%.1f ", i*i}'` )
echo -n "$DIFF "

foreach SIG ( `nawk 'BEGIN{for(i=0;i<=6;i+=0.2) printf "%.1f ", i}'` )
echo -n "$SIG "

fft HKLIN FH.mtz MAPOUT patt.map << EOF-fft >> $logfile
PATTERSON
RESO $loRES $hiRES
LABIN F1=FH SIG1=SIGFH F2=FH SIG2=SIGFH
SCALE F2 0.000001 0
EXCLUDE DIFF $DIFF
EXCLUDE SIG1 $SIG
GRID 56   88  224
EOF-fft

echo "correlate section" |\
overlapmap mapin1 site_Patt.map mapin2 patt.map |\
nawk '/Total correlation/{printf "%s ", $NF*10}'

echo "correlate section" |\
overlapmap mapin1 pdb_Patt.map mapin2 patt.map |\
nawk '/Total correlation/{printf "%s ", $NF*10}'

fft HKLIN FH.mtz MAPOUT patt.map << EOF-fft >> $logfile
PATTERSON
RESO $loRES $hiRES
LABIN F1=FH SIG1=SIGFH F2=FH SIG2=SIGFH W=W
SCALE F2 0.000001 0
EXCLUDE DIFF $DIFF
EXCLUDE SIG1 $SIG
GRID 56   88  224
EOF-fft

echo "correlate section" |\
overlapmap mapin1 site_Patt.map mapin2 patt.map |\
nawk '/Total correlation/{printf "%s ", $NF*10}'

echo "correlate section" |\
overlapmap mapin1 pdb_Patt.map mapin2 patt.map |\
nawk '/Total correlation/{printf "%s ", $NF*10}'


peakmax MAPIN patt.map XYZOUT ${tempfile}.pdb << EOF-pick |\
nawk '/Height\/Rms/{g=1} $1>1 && $1<=11 && g{++n; sum+=$4}\
$1==2 && g{second=$4} END{print sum/n,second,n/100}'
THRESHOLD RMS 3
NUMPEAKS 50
EOF-pick

end



fft HKLIN FH.mtz MAPOUT FH.map << EOF-fft
PATTERSON
RESO $loRES $hiRES
EXCLUDE DIFF $DISO
LABIN F1=FH SIG1=SIGFH F2=FH SIG2=SIGFH
EOF-fft
peakmax MAPIN patt2.map XYZOUT ${tempfile}.pdb << EOF-pick
THRESHOLD RMS 3
NUMPEAKS 50
EOF-pick
/max/jamesh/Develop/vmap patt2.map &


# calculate a "cheat" Patterson
sfall xyzin o/best_sites.pdb hklout ${tempfile}.mtz << EOF
MODE SFCALC XYZIN
SYMM P212121
RESOLUTION 1000 1.5
EOF
fft hklin ${tempfile}.mtz mapout patt.map << EOF
PATTERSON
RESO 1000 $hiRES
LABIN F1=FC
EOF

echo "patterson 0 4" |\
overlapmap mapin1 fake_Patt.map mapin2 Patterson.map |\
 nawk '/Total correlation/{print $NF}'

echo "correlate section" |\
overlapmap mapin1 fake_Patt.map mapin2 Patterson.map |\
 nawk '/Total correlation/{print $NF}'

echo "correlate section" |\
overlapmap mapin1 fake_Patt.map mapin2 maps/Patterson.map |\
 nawk '/Total correlation/{print $NF}'

